﻿using System;
using System.Configuration;
using System.Globalization;
using System.IO;
using System.Web;
using System.Xml;

namespace CustomLocalization.Web {
    public class XapHandler : IHttpHandler
    {
        private CultureInfo Culture;
        private HttpContext Context;
        private string ResourceSet;

        public bool IsReusable { get { return false; } }

        public const string BASERESOURCEPATH = "\\Resources\\";

        private static string resourceBasePath = null;

        public static string ResourceBasePath
        {
               get
               {
                   if (resourceBasePath == null)
                   {
                       resourceBasePath = ConfigurationManager.AppSettings["ResourceBasePath"];
                       if (string.IsNullOrEmpty(resourceBasePath))
                       {
                           resourceBasePath = BASERESOURCEPATH;
                       }

                       if (resourceBasePath != null && !resourceBasePath.EndsWith("\\"))
                       {
                           resourceBasePath += "\\";
                       }
                   }

                   return resourceBasePath;
               }
        }

        private static bool resourceCacheDurationSet = false;

        private static TimeSpan resourceCacheDuration;

        public static TimeSpan ResourceCacheDuration
        {
            get
            {
                if (!resourceCacheDurationSet)
                {
                    resourceCacheDurationSet = true;

                    // Set to 12 hours
                    resourceCacheDuration = new TimeSpan(12, 0, 0);

                    string strDuration = ConfigurationManager.AppSettings["ResourceCacheDuration"];
                    if (!string.IsNullOrEmpty(strDuration))
                    {
                        int duration;
                        if (int.TryParse(strDuration, out duration))
                        {
                            resourceCacheDuration = new TimeSpan(duration, 0, 0);
                        }
                    }
                }

                return resourceCacheDuration;
            }
        }

        public void ProcessRequest(HttpContext context)
        {
            Context = context;
            Culture = System.Threading.Thread.CurrentThread.CurrentCulture;

            string culture = context.Request.Params["c"];

            ResourceSet = context.Request.Params["rs"];

            if (!string.IsNullOrEmpty(culture)) {
                try
                {
                    Culture = new CultureInfo(culture);
                }
                catch (Exception ex)
                {
                    string errMsg = ex.Message;

                    // throw an error
                    Utilities.SendResponse(Context, 500,
                                           "The string " + culture + " is not recognized as a valid culture.");
                    return;
                }
            }

            string xapWithoutResources = Context.Server.MapPath(Context.Request.Path);

            if (string.IsNullOrEmpty(xapWithoutResources) || !File.Exists(xapWithoutResources))
            {
                Utilities.SendResponse(Context, 500, "The XAP file does not exist.");
                return;
            }
            else
            {
                string localizedXapFilePath = CreateLocalizedXapFile(xapWithoutResources);

                Utilities.TransmitFile(context, localizedXapFilePath, "application/x-silverlight-app", false);
            }
        }

        private string CreateLocalizedXapFile(string filePath)
        {
            FileInfo plainXap = new FileInfo(filePath);
            string localizedXapFilePath = plainXap.FullName;

            try
            {
                // Get the localized XAP file
                FileInfo localizedXap = new FileInfo(plainXap.DirectoryName + "\\" + Culture.Name + "\\" + plainXap.Name);
                
                // Get the RESX file for the locale
                FileInfo resxFile = new FileInfo(GetResourceFilePath(Context, ResourceSet, Culture.Name));

                // Check to see if the file already exists and is up to date
                if (!localizedXap.Exists || (localizedXap.LastWriteTime < plainXap.LastWriteTime) || 
                    (localizedXap.LastWriteTime < resxFile.LastWriteTime))
                {
                    if (!Directory.Exists(localizedXap.DirectoryName))
                    {
                        Directory.CreateDirectory(localizedXap.DirectoryName);
                    }
                    
                    // Copy the XAP without resources
                    localizedXap = plainXap.CopyTo(localizedXap.FullName, true);

                    // Inject the resources into the plain XAP, turning it into a localized XAP
                    if (!InjectResourceIntoXAP(localizedXap, resxFile))
                    {
                        localizedXap.Delete();
                    }
                }
                
                if (File.Exists(localizedXap.FullName))
                {
                    localizedXapFilePath = localizedXap.FullName;
                }
            }
            catch (Exception ex)
            {
                // If any error occurs, throw back the error message
                if (!File.Exists(localizedXapFilePath))
                {
                    Utilities.SendResponse(Context, 500, ex.Message);
                }
            }

            return localizedXapFilePath;
        }

        private bool InjectResourceIntoXAP(FileInfo localizedXapFile, FileInfo localizedResxFile)
        {
            if (localizedXapFile.Exists && localizedResxFile.Exists)
            {
                try
                {
                    Utilities.AddFileToZip(localizedXapFile.FullName, localizedResxFile.FullName, string.Empty);
                    return true;
                }
                catch
                {
                    return false;
                }
            }

            return false;
        }

        public static string GetResourceFilePath(HttpContext context, string resourceSet, string culture)
        {
            if (context == null) return null;
            if (string.IsNullOrEmpty(culture)) return null;

            string resxFilePath = resourceSet + "." + culture + ".resx";

            string folderPath = context.Server.MapPath(ResourceBasePath);

            FileInfo resxFile = new FileInfo(folderPath + resxFilePath);

            TimeSpan cacheDuration = ResourceCacheDuration;

            if (!resxFile.Exists || (resxFile.LastWriteTime.Add(cacheDuration) < DateTime.Now))
            {
                try
                {
                    if (!Directory.Exists(folderPath))
                    {
                        Directory.CreateDirectory(folderPath);
                    }

                    XmlDocument xmlDoc = DbResXConverter.GenerateResXFileNormalizedForCulture(resourceSet, culture);

                    xmlDoc.Save(resxFile.FullName);
                }
                catch (Exception ex)
                {
                    if (!resxFile.Exists)
                    {
                        Utilities.SendResponse(context, 500, ex.Message);
                    }
                }
            }

            return resxFile.FullName;
        }
    }
}